package com.baofoo.SMUtil;

import com.baofoo.SMUtil.Util.CryptoException;
import org.bouncycastle.jce.provider.BouncyCastleProvider;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.io.UnsupportedEncodingException;
import java.security.*;

/**
 * SM4
 * @author ximenchuixue
 * @since 2020/9/8
 */
public class SM4 {

    static {
        Security.addProvider(new BouncyCastleProvider());
    }

    private static final String ALGORITHM_NAME = "SM4";

    private static final String ALGORITHM_NAME_ECB_PADDING = "SM4/ECB/PKCS5Padding";

    private static final String ALGORITHM_NAME_CBC_PADDING = "SM4/CBC/PKCS5Padding";

    private static final int KEY_SIZE = 128;

    /**
     * 生成128bit的随机字节数组
     * @return
     *          byte[]
     * @throws CryptoException
     */
    public static byte[] generateKey0() throws CryptoException{
        try {
            KeyGenerator kg = KeyGenerator.getInstance(ALGORITHM_NAME, BouncyCastleProvider.PROVIDER_NAME);
            kg.init(KEY_SIZE, new SecureRandom());
            return kg.generateKey().getEncoded();
        } catch (GeneralSecurityException e) {
            throw new CryptoException("生成随机字节数组异常", e);
        }
    }

    /**
     * 生成128bit的16进制字符串
     * @return
     *          String 16进制
     * @throws CryptoException
     */
    public static String generateKey() throws CryptoException{
        return Hex.bytesToHex(generateKey0());
    }

    /**
     * 配置Cipher实例
     * @param algorithmName
     * @param mode
     * @param key
     * @param iv
     * @return
     * @throws CryptoException
     */
    public static Cipher configureCipher(String algorithmName, int mode, byte[] key, byte[] iv) throws CryptoException{
        try {
            Cipher cipher = Cipher.getInstance(algorithmName, BouncyCastleProvider.PROVIDER_NAME);
            Key sm4Key = new SecretKeySpec(key, ALGORITHM_NAME);
            if (null == iv) {
                cipher.init(mode, sm4Key);
            } else {
                cipher.init(mode, sm4Key, new IvParameterSpec(iv));
            }
            return cipher;
        } catch (GeneralSecurityException e) {
            throw new CryptoException("配置Cipher实例异常", e);
        }
    }

    /**
     * ECB加密
     * @param key
     *          密钥字节数组
     * @param data
     *          加密数据字节数组
     * @param iv
     *          初始向量
     * @return
     *          加密后字节数组
     * @throws CryptoException
     */
    public static byte[] encryptCbc0(byte[] key, byte[] data, byte[] iv) throws CryptoException{
        try {
            Cipher cipher = configureCipher(ALGORITHM_NAME_CBC_PADDING, Cipher.ENCRYPT_MODE, key, iv);
            return cipher.doFinal(data);
        } catch (GeneralSecurityException e) {
            throw new CryptoException("cipher doFinal异常", e);
        }
    }

    /**
     * SM4 ECB模式加密
     * @param key 密钥
     * @param data 明文数据
     * @return byte[]
     */
    public static byte[] encryptEcb(byte[] key, byte[] data) {
        try {
            Cipher cipher = configureCipher(ALGORITHM_NAME_ECB_PADDING, Cipher.ENCRYPT_MODE, key, null);
            return cipher.doFinal(data);
        } catch (GeneralSecurityException e) {
            throw new CryptoException("cipher doFinal异常", e);
        }
    }
    
    /**
     * SM4 ECB加密返String
     * @param Key
     * @param Data
     * @return
     */
    public static String encryptEcb(String Key,String Data) {
    	String ResultStr = "";
    	try {
    		byte[] TempByte = encryptEcb(Key.getBytes("UTF-8"),Data.getBytes("UTF-8"));
    		ResultStr =  Hex.bytesToHex(TempByte);
    	}catch(Exception e) {
    		throw new CryptoException("cipher doFinal异常", e);
    	}
    	return ResultStr;
    }

    /**
     * SM4 ECB模式解密
     * @param key 密钥
     * @param data 密文数据
     * @return byte[]
     */
    public static byte[] decryptEcb(byte[] key, byte[] data) {
        try {
            Cipher cipher = configureCipher(ALGORITHM_NAME_ECB_PADDING, Cipher.DECRYPT_MODE, key, null);
            return cipher.doFinal(data);
        } catch (GeneralSecurityException e) {
            throw new CryptoException("cipher doFinal异常", e);
        }
    }
    
    /**
     * SM4 ECB模式解密返回String
     * @param Key
     * @param Data
     * @return
     */
    public static String decryptEcb(String Key,String Data) {
    	String RsultStr = "";
    	try {
    		byte[] Temp_Byte = Hex.hexToBytes(Data);
    		byte[] DE_Temp_Byte = decryptEcb(Key.getBytes("UTF-8"),Temp_Byte);
    		RsultStr = new String(DE_Temp_Byte);    		
    	}catch(Exception e) {
    		throw new CryptoException("cipher doFinal异常", e);
    	}
    	return RsultStr;
    }

    /**
     * ECB加密
     * @param key
     *          密钥字节数组
     * @param data
     *          加密数据字节数组
     * @param iv
     *          初始向量
     * @return
     *          加密后string字符串
     * @throws CryptoException
     */
    public static String encryptCbc(byte[] key, byte[] data, byte[] iv) throws CryptoException{
        return Hex.bytesToHex(encryptCbc0(key, data, iv));
    }

    /**
     * ECB加密
     * @param key
     *          16进制密钥字符串
     * @param data
     *          加密数据字符串
     * @param iv
     *          初始向量
     * @return
     *          加密后string字符串
     * @throws CryptoException
     */
    public static String encryptCbc(String key, String data, byte[] iv) throws CryptoException{
        try {
            return Hex.bytesToHex(encryptCbc0(Hex.hexToBytes(key), data.getBytes("UTF-8"), iv));
        } catch (UnsupportedEncodingException e) {
            throw new CryptoException("字符编码异常", e);
        }
    }

    /**
     * ECB解密
     * @param key
     *          解密密钥字节数组
     * @param data
     *          解密数据字节数组
     * @param iv
     *          初始向量
     * @return
     *          解密后字节数组
     * @throws CryptoException
     */
    public static byte[] decryptCbc0(byte[] key, byte[] data, byte[] iv) throws CryptoException{
        try {
            Cipher cipher = configureCipher(ALGORITHM_NAME_CBC_PADDING, Cipher.DECRYPT_MODE, key, iv);
            return cipher.doFinal(data);
        } catch (GeneralSecurityException e) {
            throw new CryptoException("cipher doFinal异常", e);
        }
    }

    /**
     * ECB解密
     * @param key
     *          解密密钥字节数组
     * @param data
     *          解密数据字节数组
     * @param iv
     *          初始向量
     * @return
     *          byte[]
     * @throws CryptoException
     */
    public static byte[] decryptCbc(byte[] key, byte[] data, byte[] iv) throws CryptoException{
        return decryptCbc0(key, data, iv);
    }

    /**
     * ECB解密
     * @param key
     *          16进制密钥字符串
     * @param data
     *          解密数据字符串
     * @param iv
     *          初始向量
     * @return
     *          byte[]
     * @throws CryptoException
     */
    public static byte[] decryptCbc(String key, String data, byte[] iv) throws CryptoException{
        return decryptCbc0(Hex.hexToBytes(key), Hex.hexToBytes(data), iv);
    }
}
