package com.baofoo.SMUtil.Util;


import com.baofoo.SMUtil.SM3;
import com.baofoo.SMUtil.SM4;
import org.bouncycastle.asn1.ASN1InputStream;
import org.bouncycastle.asn1.ASN1OctetString;
import org.bouncycastle.asn1.DLSequence;
import org.bouncycastle.asn1.x509.Certificate;
import org.bouncycastle.util.encoders.Base64;
import org.bouncycastle.util.encoders.Hex;

import java.io.ByteArrayInputStream;
import java.io.IOException;

/**
 * @author ximenchuixue
 * @since 2020/9/16
 */
public class SM2CertParser extends Parser{

    /**
     * 获取公钥(16进制)
     * @param certBytes 公钥证书字节数组
     * @return 公钥(16进制)
     */
    public static String acquirePublicKeyHex(byte[] certBytes) {
        byte[] publicKey = acquirePublicKey(BufReader.read(certBytes));
        return Hex.toHexString(publicKey);
    }

    /**
     * 获取公钥
     * @param certBytes
     *          公钥证书字节数组
     * @return
     *          byte[] publicKey
     */
    public static byte[] acquirePublicKey(byte[] certBytes){
        return acquirePublicKey(BufReader.read(certBytes));
    }

    /**
     * 获取公钥
     * @param base64CertText
     *          公钥证书 base64 字符串
     * @return
     *          byte[] publicKey
     */
    public static byte[] acquirePublicKey(String base64CertText) throws CryptoException{
        ByteArrayInputStream bis = null;
        ASN1InputStream ais = null;
        try {
            bis = new ByteArrayInputStream(Base64.decode(base64CertText));
            ais = new ASN1InputStream(bis);
            DLSequence sequence = (DLSequence)ais.readObject();
            if (sequence.size() != 3){
                throw new CryptoException("SM2公钥证书错误");
            }
            Certificate certificate = Certificate.getInstance(sequence.parser());
            return certificate.getSubjectPublicKeyInfo().getPublicKeyData().getOctets();
        } catch (IOException e) {
            throw new CryptoException("获取SM2公钥异常", e);
        } finally {
            close(ais, bis);
        }
    }

    /**
     * 获取私钥(16进制)
     * @param frxBytes 私钥证书字节数组
     * @param password 私钥密码
     * @return 私钥(16进制)
     */
    public static String acquirePrivateKeyHex(byte[] frxBytes, String password) {
        byte[] privateKey = acquirePrivateKey(BufReader.read(frxBytes), password);
        return Hex.toHexString(privateKey);
    }

    /**
     * 获取私钥
     * @param pfxBytes
     *          私钥证书字节数组
     * @param password
     *          私钥密码
     * @return
     *          Byte[] privateKey
     */
    public static byte[] acquirePrivateKey(byte[] pfxBytes, String password){
        return acquirePrivateKey(BufReader.read(pfxBytes), password);
    }

    /**
     * 获取私钥
     * @param base64PfxText
     *          私钥证书 base64 字符串
     * @param password
     *          私钥密码
     * @return
     *          Byte[] privateKey
     */
    public static byte[] acquirePrivateKey(String base64PfxText, String password) throws CryptoException{
        ByteArrayInputStream bis = null;
        ASN1InputStream ais = null;
        try {
            bis = new ByteArrayInputStream(Base64.decode(base64PfxText));
            ais = new ASN1InputStream(bis);
            DLSequence sequence = (DLSequence)ais.readObject();
            if (sequence.size() != 3){
                throw new CryptoException("SM2私钥证书错误");
            }
            DLSequence pfxSequence = (DLSequence) DLSequence.getInstance(sequence.getObjectAt(1));
            byte[] privateKey = ((ASN1OctetString) pfxSequence.getObjectAt(2)).getOctets();
            byte[] encoding;
            if ((privateKey.length == 32) || (privateKey.length == 48)) {
                encoding = privateKey;
            } else {
                encoding = Base64.decode(privateKey);
            }
            byte[] hash = SM3.kdf(password.getBytes("UTF-8"));
            byte[] iv = new byte[16];
            System.arraycopy(hash, 0, iv, 0, 16);
            byte[] sm4 = new byte[16];
            System.arraycopy(hash, 16, sm4, 0, 16);
            return SM4.decryptCbc0(sm4, encoding, iv);
        } catch (IOException e) {
            throw new CryptoException("获取SM2私钥异常", e);
        } finally {
            close(ais, bis);
        }
    }
}
