package com.baofoo.SMUtil;

import com.baofoo.SMUtil.Util.CryptoException;
import com.baofoo.SMUtil.Util.FileParser;
import com.baofoo.SMUtil.Util.SM2CertParser;
import org.bouncycastle.crypto.InvalidCipherTextException;
import org.bouncycastle.crypto.engines.SM2Engine;
import org.bouncycastle.crypto.generators.ECKeyPairGenerator;
import org.bouncycastle.crypto.params.*;
import org.bouncycastle.crypto.signers.SM2Signer;
import org.bouncycastle.math.ec.ECCurve;
import org.bouncycastle.math.ec.ECPoint;
import org.bouncycastle.util.Strings;

import java.math.BigInteger;
import java.security.SecureRandom;

/**
 * @author ximenchuixue
 * @since 2020/9/8
 */
public class SM2 {

    private static String[] ECC_PARAM = {
            "FFFFFFFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000FFFFFFFFFFFFFFFF",
            "FFFFFFFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000FFFFFFFFFFFFFFFC",
            "28E9FA9E9D9F5E344D5A9E4BCF6509A7F39789F515AB8F92DDBCBD414D940E93",
            "FFFFFFFEFFFFFFFFFFFFFFFFFFFFFFFF7203DF6B21C6052B53BBF40939D54123",
            "32C4AE2C1F1981195F9904466A39C9948FE30BBFF2660BE1715A4589334C74C7",
            "BC3736A2F4F6779C59BDCEE36B692153D0A9877CC62A474002DF32E52139F0A0"
    };

    private static byte[] USER_ID = Strings.toByteArray("1234567812345678");

    private static final ECCurve ECC_CURVE;
    private static final ECDomainParameters ECC_BC_SPEC;
    private static final ECKeyPairGenerator ECC_KEY_PAIR_GENERATOR;

    static {
        BigInteger ecc_p = new BigInteger(ECC_PARAM[0], 16);
        BigInteger ecc_a = new BigInteger(ECC_PARAM[1], 16);
        BigInteger ecc_b = new BigInteger(ECC_PARAM[2], 16);
        BigInteger ecc_n = new BigInteger(ECC_PARAM[3], 16);
        BigInteger ecc_gx = new BigInteger(ECC_PARAM[4], 16);
        BigInteger ecc_gy = new BigInteger(ECC_PARAM[5], 16);

        ECC_CURVE = new ECCurve.Fp(ecc_p, ecc_a, ecc_b, ecc_n, BigInteger.ONE);
        ECPoint ecc_point_g = ECC_CURVE.createPoint(ecc_gx, ecc_gy);
        ECC_BC_SPEC = new ECDomainParameters(ECC_CURVE, ecc_point_g, ecc_n);
        ECKeyGenerationParameters generationParameters =
                new ECKeyGenerationParameters(ECC_BC_SPEC, new SecureRandom());
        ECC_KEY_PAIR_GENERATOR = new ECKeyPairGenerator();
        ECC_KEY_PAIR_GENERATOR.init(generationParameters);
    }

    /**
     * 公钥加密
     *
     * @param publicKey 公钥
     * @param data      明文
     * @return 16进制字符串
     * @throws CryptoException
     */
    public static String encryptHex(byte[] publicKey, byte[] data) throws CryptoException{
        return Hex.bytesToHex(encrypt(ECC_CURVE.decodePoint(publicKey), data));
    }
    
    /**
     * 公钥加密
     * @param PubKeyPath
     * @param Data
     * @return
     */
    public static String encryptHex(String PubKeyPath,String Data) {
    	String ResultStr = "";
    	try {
    		byte[] PubKey = SM2CertParser.acquirePublicKey(FileParser.parse(PubKeyPath));
    		byte[] Enbyte = encrypt(PubKey,Data.getBytes("UTF-8"));
    		ResultStr = Hex.bytesToHex(Enbyte);
    	}catch(Exception e) {
    		e.getStackTrace();    		
    	}
    	return ResultStr;
    }

    /**
     * SM2公钥加密
     * @param publicKey 公钥串
     * @param data 加密明文
     * @return 加密串
     */
    public static byte[] encrypt(byte[] publicKey, byte[] data) {
        return encrypt(ECC_CURVE.decodePoint(publicKey), data);
    }

    /**
     * 公钥加密
     *
     * @param base64CertText 公钥 base64 字符串
     * @param data      明文
     * @return 16进制字符串
     */
    public static String encrypt(String base64CertText, byte[] data){
        return encryptHex(SM2CertParser.acquirePublicKey(base64CertText), data);
    }

    /**
     * 公钥加密
     *
     * @param ecPoint 椭圆曲线点
     * @param data    明文
     * @return 密文
     * @throws CryptoException
     */
    public static byte[] encrypt(ECPoint ecPoint, byte[] data) throws CryptoException{
        try {
            ECPublicKeyParameters publicKeyParameters = new ECPublicKeyParameters(ecPoint, ECC_BC_SPEC);
            ParametersWithRandom parametersWithRandom = new ParametersWithRandom(publicKeyParameters);
            SM2Engine sm2Engine = new SM2Engine();
            sm2Engine.init(true, parametersWithRandom);
            return sm2Engine.processBlock(data, 0, data.length);
        } catch (InvalidCipherTextException e) {
            throw new CryptoException("SM2公钥加密异常", e);
        }
    }

    /**
     * 私钥解密
     *
     * @param pfxBytes 私钥字节
     * @param password 私钥密码
     * @param data       密文
     * @return 明文
     */
    public static byte[] decrypt(byte[] pfxBytes, String password, String data) {
        return decrypt(SM2CertParser.acquirePrivateKey(pfxBytes, password), Hex.hexToBytes(data));
    }
    
    /**
     * 私钥解密
     * @param Secret
     * @param Private_Path
     * @param Passwd
     * @return
     */
    public static String decryptStr(String Secret,String Private_Path,String Passwd) {
    	byte[] Key_B = SM2CertParser.acquirePrivateKey(FileParser.parse(Private_Path), Passwd);
    	byte[] Plaintext = decrypt(Key_B,Hex.hexToBytes(Secret));
    	String Result = new String(Plaintext);
    	return Result;
    }

    /**
     * 私钥解密
     *
     * @param base64PfxText base64私钥字符串
     * @param password 私钥密码
     * @param data       密文
     * @return 明文
     */
    public static byte[] decrypt(String base64PfxText, String password, String data) {
        return decrypt(SM2CertParser.acquirePrivateKey(base64PfxText, password), Hex.hexToBytes(data));
    }

    /**
     * 私钥解密
     *
     * @param privateKey 私钥
     * @param data       密文
     * @return 明文
     */
    public static byte[] decrypt(byte[] privateKey, byte[] data) {
        return decrypt0(new BigInteger(1, privateKey), data);
    }


    /**
     * 私钥解密
     *
     * @param privateKey 私钥
     * @param data       密文
     * @return 明文
     * @throws CryptoException
     */
    public static byte[] decrypt0(BigInteger privateKey, byte[] data) throws CryptoException{
        try {
            ECPrivateKeyParameters privateKeyParameters = new ECPrivateKeyParameters(privateKey, ECC_BC_SPEC);
            SM2Engine sm2Engine = new SM2Engine();
            sm2Engine.init(false, privateKeyParameters);
            return sm2Engine.processBlock(data, 0, data.length);
        } catch (InvalidCipherTextException e) {
            throw new CryptoException("SM2私钥解密异常", e);
        }
    }

    /**
     * 私钥签名, userId:1234567812345678
     * @param privateKey SM2 私钥
     * @param data 签名数据
     * @return 签名串
     */
    public static byte[] sign(byte[] privateKey, byte[] data) {
        return sign0(USER_ID, privateKey, data);
    }
    


    /**
     * 私钥签名
     *
     * @param id     用户标识
     * @param pfxBytes 私钥字节
     * @param password 私钥密码
     * @param data     报文字节
     * @return 签名
     * @throws CryptoException
     */
    public static byte[] sign(byte[] id, byte[] pfxBytes, String password, byte[] data) throws CryptoException {
        return sign0(id, SM2CertParser.acquirePrivateKey(pfxBytes, password), data);
    }

    /**
     * 私钥签名
     *
     * @param id     用户标识
     * @param base64PfxText base64私钥字符串
     * @param password 私钥密码
     * @param data     报文字节
     * @return 签名
     * @throws CryptoException
     */
    public static byte[] sign(byte[] id, String base64PfxText, String password, byte[] data) throws CryptoException {
        return sign0(id, SM2CertParser.acquirePrivateKey(base64PfxText, password), data);
    }

    /**
     * 私钥签名
     *
     * @param id     用户标识
     * @param privateKey 私钥
     * @param data       明文
     * @return 签名
     * @throws CryptoException
     */
    public static byte[] sign0(byte[] id, byte[] privateKey, byte[] data) throws CryptoException {
        try {
            ECPrivateKeyParameters privateKeyParameters = new ECPrivateKeyParameters(new BigInteger(1, privateKey), ECC_BC_SPEC);
            ParametersWithID parametersWithID = new ParametersWithID(privateKeyParameters, id);
            SM2Signer sm2Signer = new SM2Signer();
            sm2Signer.init(true, parametersWithID);
            sm2Signer.update(data, 0, data.length);
            return sm2Signer.generateSignature();
        } catch (org.bouncycastle.crypto.CryptoException e) {
            throw new CryptoException("SM2私钥签名异常", e);
        }
    }
    
    public static String sign(String Terminalid,String Private_Key_Path,String Passwd,String Data_Str) {
    	String Result_Str = "";
    	try {
    		byte[] P_key = SM2CertParser.acquirePrivateKey(FileParser.parse(Private_Key_Path), Passwd);
    		byte[] Temp_Result_Str = sign0(Terminalid.getBytes("UTF-8"),P_key,Data_Str.getBytes("UTF-8"));
    		Result_Str = Hex.bytesToHex(Temp_Result_Str);
    	}catch(Exception e) {
    		e.getStackTrace();
    	}
    	return Result_Str;
    }

    /**
     * 校验签名
     * @param publicKey 公钥
     * @param sourceData 签名数据
     * @param signData 签名串
     * @return boolean
     */
    public static boolean verify(byte[] publicKey, byte[] sourceData, byte[] signData) {
       return verify0(USER_ID, publicKey, sourceData, signData);
    }

    /**
     * 公钥验签
     *
     * @param id     用户标识
     * @param publicKey  公钥
     * @param sourceData 源数据
     * @param signData   签名
     * @return 验签结果
     */
    public static boolean verify(byte[] id, byte[] publicKey, byte[] sourceData, String signData) {
        return verify0(id, publicKey, sourceData, Hex.hexToBytes(signData));
    }

    /**
     * 公钥验签
     *
     * @param id     用户标识
     * @param base64CertText  公钥 base64 字符串
     * @param sourceData 源数据
     * @param signData   签名
     * @return 验签结果
     */
    public static boolean verify(byte[] id, String base64CertText, byte[] sourceData, String signData) {
        return verify0(id, SM2CertParser.acquirePublicKey(base64CertText), sourceData, Hex.hexToBytes(signData));
    }


    /**
     * 公钥验签
     *
     * @param id     用户标识
     * @param publicKey   公钥
     * @param sourceData 源数据
     * @param signData   签名
     * @return 验签结果
     */
    public static boolean verify0(byte[] id, byte[] publicKey, byte[] sourceData, byte[] signData) {
        ECPublicKeyParameters publicKeyParameters = new ECPublicKeyParameters(ECC_CURVE.decodePoint(publicKey), ECC_BC_SPEC);
        ParametersWithID parametersWithID = new ParametersWithID(publicKeyParameters, id);
        SM2Signer sm2Signer = new SM2Signer();
        sm2Signer.init(false, parametersWithID);
        sm2Signer.update(sourceData, 0, sourceData.length);
        return sm2Signer.verifySignature(signData);
    }
    
    /**
     * 公钥验签
     * @param Terminalid
     * @param public_Key_Path
     * @param Source_Data
     * @param Sign_Data
     * @return
     */
    public static boolean verify0(String Terminalid,String public_Key_Path,String Source_Data,String Sign_Data) {    	
    	try {
    		byte[] Pu_key = SM2CertParser.acquirePublicKey(FileParser.parse(public_Key_Path));
    		return verify0(Terminalid.getBytes("UTF-8"),Pu_key,Source_Data.getBytes("UTF-8"),Hex.hexToBytes(Sign_Data));
    	}catch(Exception e) {
    		e.getStackTrace();
    	}
    	return false;
    }

    /**
     * 公钥验签
     * @param id 用户标识
     * @param ecPoint ECPoint
     * @param sourceData 源数据
     * @param signData 签名
     * @return 验签结果
     */
    public static boolean verify(byte[] id, ECPoint ecPoint, byte[] sourceData, byte[] signData) {
        ECPublicKeyParameters publicKeyParameters = new ECPublicKeyParameters(ecPoint, ECC_BC_SPEC);
        ParametersWithID parametersWithID = new ParametersWithID(publicKeyParameters, id);
        SM2Signer sm2Signer = new SM2Signer();
        sm2Signer.init(false, parametersWithID);
        sm2Signer.update(sourceData, 0, sourceData.length);
        return sm2Signer.verifySignature(signData);
    }

    /**
     * 获取 ECKeyPairGenerator
     *
     * @return ECKeyPairGenerator
     */
    public static ECKeyPairGenerator keyPairGenerator() {
        return ECC_KEY_PAIR_GENERATOR;
    }

    public static ECCurve getEccCurve(){
        return ECC_CURVE;
    }
}
